commit 6980497f7d7f4d17b918a7a433aa904943a4bb97
Author: Heikki Linnakangas <heikki.linnakangas@iki.fi>
Date:   Mon Mar 25 11:02:55 2013 +0200

    Add a server version check to pg_basebackup and pg_receivexlog.
    
    These programs don't work against 9.0 or earlier servers, so check that when
    the connection is made. That's better than a cryptic error message you got
    before.
    
    Also, these programs won't work with a 9.3 server, because the WAL streaming
    protocol was changed in a non-backwards-compatible way. As a general rule,
    we don't make any guarantee that an old client will work with a new server,
    so check that. However, allow a 9.1 client to connect to a 9.2 server, to
    avoid breaking environments that currently work; a 9.1 client happens to
    work with a 9.2 server, even though we didn't make any great effort to
    ensure that.
    
    This patch is for the 9.1 and 9.2 branches, I'll commit a similar patch to
    master later. Although this isn't a critical bug fix, it seems safe enough
    to back-patch. The error message you got when connecting to a 9.3devel
    server without this patch was cryptic enough to warrant backpatching.

diff --git a/src/bin/pg_basebackup/pg_basebackup.c b/src/bin/pg_basebackup/pg_basebackup.c
index bddd371..19cc9e8 100644
--- a/src/bin/pg_basebackup/pg_basebackup.c
+++ b/src/bin/pg_basebackup/pg_basebackup.c
@@ -947,6 +947,9 @@ BaseBackup(void)
 	int			i;
 	char		xlogstart[64];
 	char		xlogend[64];
+	int			minServerMajor,
+				maxServerMajor;
+	int			serverMajor;
 
 	/*
 	 * Connect in replication mode to the server
@@ -957,6 +960,21 @@ BaseBackup(void)
 		exit(1);
 
 	/*
+	 * Check server version. BASE_BACKUP command was introduced in 9.1, so
+	 * we can't work with servers older than 9.1. And we don't support servers
+	 * newer than the client.
+	 */
+	minServerMajor = 901;
+	maxServerMajor = PG_VERSION_NUM / 100;
+	serverMajor = PQserverVersion(conn) / 100;
+	if (serverMajor < minServerMajor || serverMajor > maxServerMajor)
+	{
+		fprintf(stderr, _("%s: unsupported server version %s\n"),
+				progname, PQparameterStatus(conn, "server_version"));
+		disconnect_and_exit(1);
+	}
+
+	/*
 	 * Run IDENTIFY_SYSTEM so we can get the timeline
 	 */
 	res = PQexec(conn, "IDENTIFY_SYSTEM");
diff --git a/src/bin/pg_basebackup/pg_receivexlog.c b/src/bin/pg_basebackup/pg_receivexlog.c
index 4d91add..b7df693 100644
--- a/src/bin/pg_basebackup/pg_receivexlog.c
+++ b/src/bin/pg_basebackup/pg_receivexlog.c
@@ -220,6 +220,9 @@ StreamLog(void)
 	PGresult   *res;
 	uint32		timeline;
 	XLogRecPtr	startpos;
+	int			minServerMajor,
+				maxServerMajor;
+	int			serverMajor;
 
 	/*
 	 * Connect in replication mode to the server
@@ -230,6 +233,21 @@ StreamLog(void)
 		return;
 
 	/*
+	 * Check server version. IDENTIFY_SYSTEM didn't return the current xlog
+	 * position before 9.1, so we can't work with servers older than 9.1. And
+	 * we don't support servers newer than the client.
+	 */
+	minServerMajor = 901;
+	maxServerMajor = PG_VERSION_NUM / 100;
+	serverMajor = PQserverVersion(conn) / 100;
+	if (serverMajor < minServerMajor || serverMajor > maxServerMajor)
+	{
+		fprintf(stderr, _("%s: unsupported server version %s\n"),
+				progname, PQparameterStatus(conn, "server_version"));
+		disconnect_and_exit(1);
+	}
+
+	/*
 	 * Run IDENTIFY_SYSTEM so we can get the timeline and current xlog
 	 * position.
 	 */
